(* ------------------------------------------------------------- *)
(* TVID: Three-loop Virtual Integrals from Dispersion relations  *)
(* version 2.0 (Aug 23, 2019)                                    *)
(* ------------------------------------------------------------- *)
(* i3.m                                                          *)
(* Ayres Freitas (afreitas@pitt.edu)			         *)
(* last revision 06.08.19                                        *)
(* ------------------------------------------------------------- *)
(* Mathematica file for subtraction of divergent pieces from     *)
(* 3-loop vacuum master integrals U4, U5, U6;          	         *)
(* expansion in eps=(4-d)/2; and analytical formulae for         *)
(* special cases with few different masses;                      *)
(* also loads uself.m for 3-loop self-energy integrals 	         *)
(* ------------------------------------------------------------- *)
(* References:                                                   *)
(*   [1] = Freitas, arXiv:1609.09159, JHEP 1611 (2016) 145       *)
(*   [2] = Bauberger, Freitas, Wiegand, arXiv:1908.09887         *)
(* ------------------------------------------------------------- *)

lambda[m1s_,m2s_,m3s_] :=
        m1s^2 + m2s^2 + m3s^2 - 2*(m1s*m2s + m1s*m3s + m2s*m3s);

$lpfac = Exp[EulerGamma*$eps];

(* Note: in this implementation x,y,u,v must be avoided in input! *)

trepl = {
  A0[0] -> 0,
  A0[ms_] :> -$lpfac * Gamma[(2-$D)/2]*ms^($D/2-1),
  T3[M1s_,M2s_,M3s_] :> Sort[T3[M1s,M2s,M3s]] /; !OrderedQ[{M1s,M2s,M3s}],
  T3[0,0,0] -> 0,
  T3[0,0,m3s_] :> $lpfac^2 * m3s^(1-2*$eps) *
      Gamma[1-$eps]*Gamma[$eps]*Gamma[-1+2*$eps]/($eps-1),
  T3[0,m2s_,m3s_] :> $lpfac^2 * m3s^(1-2*$eps) *
    Gamma[1+$eps]^2/(2*(1-$eps)*(1-2*$eps)) *
    ((1+y)/$eps^2 - (2*y*Log[y])/$eps + 
     (y*Log[y]^2 + 2*(1-y)*PolyLog[2, 1-y]) +
     T3Idelta[0,m2s,m3s]*$eps /. y->m2s/m3s), 
  T3[m1s_,m2s_,m3s_] :> $lpfac^2 * m3s^(1-2*$eps) *
    Gamma[1+$eps]^2/(2*(1-$eps)*(1-2*$eps)) *
    ((1+x+y)/$eps^2 - 2*(x*Log[x] + y*Log[y])/$eps +
     (x*Log[x]^2 + y*Log[y]^2 - (1-x-y)*Log[x]*Log[y] +
      sla*(2*Log[u]*Log[v] - Log[x]*Log[y] -
	   2*PolyLog[2,u] - 2*PolyLog[2,v] + Pi^2/3)) +
     T3Idelta[m1s,m2s,m3s]*$eps /.
    {u -> (1+x-y-sla)/2, v -> (1-x+y-sla)/2} /.
     sla -> Sqrt[lambda[1,x,y]] /. {x->m1s/m3s, y->m2s/m3s})  
};

t134ep = (y*Log[y]*(3*Log[x]^2 + 3*Log[x]*Log[y] + 2*Log[y]^2))/6 + 
 (x*Log[x]*(2*Log[x]^2 + 3*Log[x]*Log[y] + 3*Log[y]^2))/6 - 
 (Log[x]*Log[y]*Log[x*y])/2 + l*((Log[x]^2*Log[y])/2 + (Log[x]*Log[y]^2)/2 + 
   (2*Pi^2*Log[1 - w*z])/3 + Log[x]^2*Log[1 - w*z] + 
   2*Log[x]*Log[y]*Log[1 - w*z] + Log[y]^2*Log[1 - w*z] + 
   2*Log[x]*Log[1 - w*z]^2 + 2*Log[y]*Log[1 - w*z]^2 - 
   2*Log[w*z]*Log[1 - w*z]^2 + (4*Log[1 - w*z]^3)/3 + 
   (2*Pi^2*Log[(1 - w*z)/((-1 + w)*(-1 + z))])/3 + 
   (4*Log[(1 - w*z)/((-1 + w)*(-1 + z))]^3)/3 - 
   (2*Pi^2*Log[(-1 + w*z)/(-1 + w)])/3 - Log[x]^2*Log[(-1 + w*z)/(-1 + w)] - 
   2*Log[x]*Log[(-1 + w*z)/(-1 + w)]^2 + 2*Log[((-1 + l - x + y)*(-1 + z))/2]*
    Log[(-1 + w*z)/(-1 + w)]^2 - (4*Log[(-1 + w*z)/(-1 + w)]^3)/3 - 
   (2*Pi^2*Log[(-1 + w*z)/(-1 + z)])/3 - Log[y]^2*Log[(-1 + w*z)/(-1 + z)] + 
   2*Log[((-1 + w)*(-1 + l + x - y))/2]*Log[(-1 + w*z)/(-1 + z)]^2 - 
   2*Log[y]*Log[(-1 + w*z)/(-1 + z)]^2 - (4*Log[(-1 + w*z)/(-1 + z)]^3)/3 - 
   2*Log[y]*PolyLog[2, ((-1 + w)*(-1 + l + x - y))/2] - 
   2*Log[x]*PolyLog[2, ((-1 + l - x + y)*(-1 + z))/2] + 
   2*Log[x*y]*PolyLog[2, w*z] + 2*PolyLog[3, ((-1 + w)*(-1 + l + x - y))/2] + 
   2*PolyLog[3, ((-1 + l - x + y)*(-1 + z))/2] - 2*PolyLog[3, w*z] - 
   4*PolyLog[3, 1 - w*z] + 4*PolyLog[3, (-1 + w*z)/(-1 + w)] + 
   4*PolyLog[3, (-1 + w*z)/(-1 + z)] - 2*Zeta[3]);

t134ep0 = -(y*Log[y]^3)/3 + (-1 + y)*(-(Pi^2*Log[y])/3 + Log[1 - y]*Log[y]^2 + 
   2*Log[y]*PolyLog[2, 1 - y] + 4*PolyLog[3, 1 - y] + 2*PolyLog[3, y] - 
   2*Zeta[3]);

trepl2 = {
  T3Idelta[0,M2s_,M3s_] :> (t134ep0 /. y->M2s/M3s),
  T3Idelta[M1s_,M2s_,M3s_] :> (-t134ep //.
                 {w -> 1/(2y)*(-1+x+y+l), z -> 1/(2x)*(-1+x+y+l)} /.
                  l -> Sqrt[lambda[1,x,y]] /. {x->M1s/M3s, y->M2s/M3s})
};


usort = { U4[m1s_,mms__] :> Prepend[Sort[U4[mms]], m1s],
(*          U5[m1s_,m2s_,m3s_,0,m5s_] :> U5[m1s,m2s,0,m3s,m5s] /; m3s=!=0,
	  U5[0,m2s_,m3s_,m4s_,m5s_] :> U5[m2s,0,m3s,m4s,m5s] /; m2s=!=0,
	  U5[m1s_,m2s_,0,m4s_,m5s_] :> U5[m4s,0,m2s,m1s,m5s] /; m1s=!=0, *)
	  U5[m1s_,m2s_,m3s_,m4s_,m5s_] :> (Append[Sort[{Sort[{m1s,m2s}],
	  					  Sort[{m3s,m4s}]}], m5s] /.
		{{xx1_,xx2_},{xx3_,xx4_},xx5_} :> U5[xx2,xx1,xx3,xx4,xx5]),
          U6[m1s_,m2s_,m3s_,m4s_,m5s_,m6s_] :> (Last[Sort[
	    {{m1s,m2s,m3s,m4s,m5s,m6s}, {m1s,m4s,m3s,m2s,m6s,m5s}, 
	     {m1s,m5s,m3s,m6s,m2s,m4s}, {m1s,m6s,m3s,m5s,m4s,m2s}, 
	     {m2s,m1s,m4s,m3s,m5s,m6s}, {m2s,m3s,m4s,m1s,m6s,m5s}, 
	     {m2s,m5s,m4s,m6s,m1s,m3s}, {m2s,m6s,m4s,m5s,m3s,m1s}, 
	     {m3s,m2s,m1s,m4s,m6s,m5s}, {m3s,m4s,m1s,m2s,m5s,m6s}, 
	     {m3s,m5s,m1s,m6s,m4s,m2s}, {m3s,m6s,m1s,m5s,m2s,m4s}, 
	     {m4s,m1s,m2s,m3s,m6s,m5s}, {m4s,m3s,m2s,m1s,m5s,m6s}, 
	     {m4s,m5s,m2s,m6s,m3s,m1s}, {m4s,m6s,m2s,m5s,m1s,m3s}, 
	     {m5s,m1s,m6s,m3s,m2s,m4s}, {m5s,m2s,m6s,m4s,m1s,m3s}, 
	     {m5s,m3s,m6s,m1s,m4s,m2s}, {m5s,m4s,m6s,m2s,m3s,m1s}, 
	     {m6s,m1s,m5s,m3s,m4s,m2s}, {m6s,m2s,m5s,m4s,m3s,m1s}, 
	     {m6s,m3s,m5s,m1s,m2s,m4s}, {m6s,m4s,m5s,m2s,m1s,m3s}}]] /.
	    {xx1_,xx2_,xx3_,xx4_,xx5_,xx6_} :> U6[xx2,xx6,xx4,xx5,xx3,xx1])
	};

urepl = {
  U4[ms_,0,0,0] :> 2*Gamma[1-$eps]^2*Gamma[-2+2*$eps]*Gamma[-1+3*$eps]*
                    ms^(1-3*$eps) * Exp[3*EulerGamma*$eps] /; !PossibleZeroQ[ms],
  U4[0,0,0,ms_] :> -ms^(1-3*$eps) * (
    1/(6*$eps^3) + 1/(3*$eps^2) + (52 + 5*Pi^2)/(24*$eps) + 
     (40 + 5*Pi^2 - 11*Zeta[3])/6) /; !PossibleZeroQ[ms],
  U4[ms_,0,0,ms_] :> ms^(1-3*$eps) * (
    1/(3*$eps^3) + 2/(3*$eps^2) + (4 + Pi^2)/(12*$eps) + 
     (-20 + Pi^2 + 14*Zeta[3])/6) /; !PossibleZeroQ[ms],
  U4[m1s_,0,0,m4s_] :> m4s^(1-3*$eps) * (
    1/(3*$eps^3) + (5 - x - 3*Log[x])/(6*$eps^2) + 
     (16 + Pi^2 - 12*x + 6*(-3 + x)*Log[x] + 3*Log[x]^2)/(12*$eps) + 
     (20 + 5*Pi^2 - 100*x - Pi^2*x - 84*Log[x] - 11*Pi^2*Log[x] + 72*x*Log[x] + 
       18*Log[x]^2 - 6*x*Log[x]^2 + 24*Log[1-x]*Log[x]^2 - 
       2*Log[x]^3 + 24*(-1 + x + Log[x])*PolyLog[2, 1-x] + 
       48*PolyLog[3, x] + 8*Zeta[3])/24 /. x->m1s/m4s) /; !PossibleZeroQ[m1s*m4s],
  U4[0,0,ms_,ms_] :> -ms^(1-3*$eps) * (
    1/(3*$eps^3) + 4/(3*$eps^2) + (52 + Pi^2)/(12*$eps) + 
     (40 + Pi^2 + 7*Zeta[3])/3) /; !PossibleZeroQ[ms], 
  U4[0,0,m3s_,m4s_] :> -m4s^(1-3*$eps) * (
    (1+x)/(6*$eps^3) + (4 + 4*x - 3*x*Log[x])/(6*$eps^2) + 
     ((52 + Pi^2)*(1+x) - 48*x*Log[x] + 6*x*Log[x]^2 + 
       24*(1-x)*PolyLog[2, 1-x])/(24*$eps) + 
     (4*(40 + Pi^2)*(1+x) + 4*Zeta[3]*(13 - 11*x) - 
       (156*x - 4*Pi^2 + 15*Pi^2*x)*Log[x] +
       12*(2*x - (1 - 3*x)*Log[1-x])*Log[x]^2 - 2*x*Log[x]^3 + 
       24*(4*(1-x) - (1 - 2*x)*Log[x])*PolyLog[2, 1-x] - 
       96*(1-x)*PolyLog[3, 1-x] - 24*(1 - 3*x)*PolyLog[3, x])/24 /. 
     x->m3s/m4s) /; !PossibleZeroQ[m3s*m4s],
  U4[m1s_,0,m3s_,m4s_] :> (m3s + m4s)/(3*$eps^3) + 
   (-m1s + 5*m3s + 5*m4s - 3*(m3s + m4s)*Log[m1s] - 
     3*m3s*Log[m3s] - 3*m4s*Log[m4s])/(6*$eps^2) + 
   (6*m1s*(-2 + Log[m1s]) + (m3s + m4s)*(16 + Pi^2 - 18*Log[m1s] + 
       3*Log[m1s]^2) + 12*(-1 + Log[m1s])*(m3s*Log[m3s] + m4s*Log[m4s]) + 
     3*(m3s*Log[m3s]^2 + m4s*Log[m4s]^2))/(12*$eps) + 
   (-4*m1s*Pi^2 + 5*(m3s + m4s)*(4 + Pi^2) + m1s*(-100 + 7*Pi^2) + 
     72*m1s*Log[m1s] + (m3s + m4s)*(-84 + 5*Pi^2)*Log[m1s] + 2*m3s*Log[m1s]^3 + 
     2*m4s*Log[m1s]^3 - 12*m3s*Log[m3s] - 11*m3s*Pi^2*Log[m3s] + 
     12*m3s*Log[m3s]^2 - 2*m3s*Log[m3s]^3 + 48*m3s*Log[m1s]*Log0[1 - m3s/m1s] - 
     48*m3s*Log[m3s]*Log0[1 - m3s/m1s] + 24*m3s*Log[m3s]^2*Log0[1 - m3s/m1s] + 
     48*m3s*Log[m3s/m1s]*Log0[1 - m3s/m1s] - 12*m3s*Log[m1s]*Log[m3s]*
      (-4 + Log[m3s] + 4*Log0[1 - m3s/m1s]) - 12*m4s*Log[m4s] - 
     11*m4s*Pi^2*Log[m4s] + 12*m4s*Log[m4s]^2 - 2*m4s*Log[m4s]^3 + 
     48*m4s*Log[m1s]*Log0[1 - m4s/m1s] - 48*m4s*Log[m4s]*Log0[1 - m4s/m1s] + 
     24*m4s*Log[m4s]^2*Log0[1 - m4s/m1s] + 48*m4s*Log[m4s/m1s]*Log0[1 - m4s/m1s] - 
     12*m4s*Log[m1s]*Log[m4s]*(-4 + Log[m4s] + 4*Log0[1 - m4s/m1s]) - 
     6*Log[m1s]^2*(3*m1s - 5*m3s - 5*m4s + 4*m3s*Log[m3s] - 
       4*m3s*Log0[1 - m3s/m1s] + 4*m4s*Log[m4s] - 4*m4s*Log0[1 - m4s/m1s]) - 
     24*(m1s - m3s + m3s*Log[m1s] - m3s*Log[m3s])*PolyLog[2, 1 - m3s/m1s] - 
     24*(m1s - m4s + m4s*Log[m1s] - m4s*Log[m4s])*PolyLog[2, 1 - m4s/m1s] + 
     48*m3s*PolyLog[3, m3s/m1s] + 48*m4s*PolyLog[3, m4s/m1s] + 8*m3s*Zeta[3] + 
     8*m4s*Zeta[3])/24 + U4sub[m1s, 0, m3s, m4s] /; !PossibleZeroQ[m1s*m3s*m4s],
  U4[0,m2s_,m3s_,m4s_] :> (1-$D/2)*A0[m4s]/m4s*T3[m2s, m3s, m4s] +
   (((m2s+m3s+m4s)/3)/$eps^3 +
    ((m2s+m3s+m4s)*(5/6-Log[m4s]/2) -
     (m2s*Log[m2s]+m3s*Log[m3s]+m4s*Log[m4s])/2)/$eps^2 +
    ((m2s+m3s+m4s)*(4/3+Pi^2/12-3/2*Log[m4s]+1/4*Log[m4s]^2) +
     (Log[m4s]-1)*(m2s*Log[m2s]+m3s*Log[m3s]+m4s*Log[m4s]) +
     (m2s*Log[m2s]^2+m3s*Log[m3s]^2+m4s*Log[m4s]^2)/4)/$eps +
    ((m2s + m3s + m4s)*(20 + Pi^2 + 8*Zeta[3]) + (-12 + Pi^2)*m4s*Log[m4s] + 
     (m2s + m3s + m4s)*(-84 - 7*Pi^2)*Log[m4s] - 
     34*m4s*Log[m4s]^3 - 2*(m2s + m3s + m4s)*Log[m4s]^3 +
     (-12 + Pi^2)*m2s*Log[m2s] + 2*m2s*Log[m2s]^3 + 
     (-12 + Pi^2)*m3s*Log[m3s] + 2*m3s*Log[m3s]^3 + Log[m4s]*
     (72*m2s*Log[m2s] -24*m2s*Log[m2s]^2 + 72*m3s*Log[m3s] -24*m3s*Log[m3s]^2) +
     Log[m4s]^2*(18*(5*m4s+m3s+m2s) - 12*m2s*Log[m2s] - 12*m3s*Log[m3s]))/24 +
     U4sub0[m4s,m3s,m2s]) /; !PossibleZeroQ[m2s*m3s*m4s],
  U4[m1s_,m2s_,m3s_,m4s_] :> (((m2s+m3s+m4s)/3)/$eps^3 +
		 ((m2s+m3s+m4s)*(5/6-Log[m1s]/2) - m1s/6 -
		  (m2s*Log[m2s]+m3s*Log[m3s]+m4s*Log[m4s])/2)/$eps^2 +
		 ((m2s+m3s+m4s)*(4/3+Pi^2/12-3/2*Log[m1s]+1/4*Log[m1s]^2) -
		  m1s*(1-Log[m1s]/2) +
		  (Log[m1s]-1)*(m2s*Log[m2s]+m3s*Log[m3s]+m4s*Log[m4s]) +
		  (m2s*Log[m2s]^2+m3s*Log[m3s]^2+m4s*Log[m4s]^2)/4)/$eps +
   (5*(m2s + m3s + m4s)*(4 + Pi^2) + m1s*(-100 + 7*Pi^2) + 72*m1s*Log[m1s] + 
     (m2s + m3s + m4s)*(-84 + 5*Pi^2)*Log[m1s] + 2*(m2s + m3s + m4s)*Log[m1s]^3 - 
     12*m2s*Log[m2s] - 11*m2s*Pi^2*Log[m2s] + 12*m2s*Log[m2s]^2 - 2*m2s*Log[m2s]^3 - 
     12*m3s*Log[m3s] - 11*m3s*Pi^2*Log[m3s] + 12*m3s*Log[m3s]^2 - 2*m3s*Log[m3s]^3 - 
     12*m4s*Log[m4s] - 11*m4s*Pi^2*Log[m4s] + 12*m4s*Log[m4s]^2 - 2*m4s*Log[m4s]^3 + 
     48*m2s*Log[m1s]*Log0[1 - m2s/m1s] - 48*m2s*Log[m2s]*Log0[1 - m2s/m1s] + 
     24*m2s*Log[m2s]^2*Log0[1 - m2s/m1s] + 48*m2s*Log[m2s/m1s]*Log0[1 - m2s/m1s] - 
     12*m2s*Log[m1s]*Log[m2s]*(-4 + Log[m2s] + 4*Log0[1 - m2s/m1s]) + 
     48*m3s*Log[m1s]*Log0[1 - m3s/m1s] - 48*m3s*Log[m3s]*Log0[1 - m3s/m1s] + 
     24*m3s*Log[m3s]^2*Log0[1 - m3s/m1s] + 48*m3s*Log[m3s/m1s]*Log0[1 - m3s/m1s] - 
     12*m3s*Log[m1s]*Log[m3s]*(-4 + Log[m3s] + 4*Log0[1 - m3s/m1s]) - 
     6*Log[m1s]^2*(3*m1s - 5*(m2s + m3s + m4s) + 4*m2s*Log[m2s] + 4*m3s*Log[m3s] - 
       4*m2s*Log0[1 - m2s/m1s] - 4*m3s*Log0[1 - m3s/m1s] + 
       4*m4s*(Log[m4s] - Log0[1 - m4s/m1s])) + 48*m4s*Log[m1s]*Log0[1 - m4s/m1s] - 
     48*m4s*Log[m4s]*Log0[1 - m4s/m1s] + 24*m4s*Log[m4s]^2*Log0[1 - m4s/m1s] + 
     48*m4s*Log[m4s/m1s]*Log0[1 - m4s/m1s] - 12*m4s*Log[m1s]*Log[m4s]*
      (-4 + Log[m4s] + 4*Log0[1 - m4s/m1s]) - 
     24*(m1s - m2s + m2s*Log[m1s] - m2s*Log[m2s])*PolyLog[2, 1 - m2s/m1s] - 
     24*(m1s - m3s + m3s*Log[m1s] - m3s*Log[m3s])*PolyLog[2, 1 - m3s/m1s] - 
     24*(m1s - m4s + m4s*Log[m1s] - m4s*Log[m4s])*PolyLog[2, 1 - m4s/m1s] + 
     48*m2s*PolyLog[3, m2s/m1s] + 48*m3s*PolyLog[3, m3s/m1s] + 
     48*m4s*PolyLog[3, m4s/m1s] + 8*(m2s + m3s + m4s)*Zeta[3])/24 +
		   U4sub[m1s,m2s,m3s,m4s]) /; !PossibleZeroQ[m1s*m2s*m3s*m4s],
  U5[0,0,0,0,ms_] :> ms^(1-3*$eps) * (
    1/(3*$eps^3) + 5/(3*$eps^2) + (68 + 5*Pi^2)/(12*$eps) + 
     (196 + 25*Pi^2 - 20*Zeta[3])/12) /; !PossibleZeroQ[ms],
  U5[0,0,0,ms_,0] :> ms^(1-3*$eps) * (
    1/(6*$eps^3) + 1/$eps^2 + (100 + 5*Pi^2)/(24*$eps) +
     (15 + 5*Pi^2/4 - 11*Zeta[3]/6)) /; !PossibleZeroQ[ms],
  U5[0,0,0,m4s_,m5s_] :> m5s^(1-3*$eps) * (
    (2+x)/(6*$eps^3) + (10 + 6*x - 3*x*Log[x])/(6*$eps^2) + 
     (136 + 6*Pi^2 + 100*x + Pi^2*x - 72*x*Log[x] + 6*x*Log[x]^2 + 
       24*(1-x)*PolyLog[2, 1-x])/(24*$eps) + 
     (392 + 30*Pi^2 + 360*x + 10*Pi^2*x - 4*Zeta[3]*(4 + 5*x) - 
       (300*x - 4*Pi^2 + 11*Pi^2*x)*Log[x] +
       12*(-2 + 6*x - (1-x)*Log0[1-x])*Log[x]^2 + (8 - 10*x)*Log[x]^3 + 
       24*(3*(1-x) - (1-x)*Log[x])*PolyLog[2, 1-x] - 
       24*(1-x)*(2 - Log[x])*PolyLog[2, 1-1/x] -
       24*(1-x)*(PolyLog[3, 1-x] + PolyLog[3, x] - PolyLog[3, 1-1/x]))/24 /. 
     x->m4s/m5s) /; !PossibleZeroQ[m4s*m5s],
  U5[0,0,m3s_,m4s_,0] :> m4s^(1-3*$eps) * (
    (1+x)/(6*$eps^3) + (1 + x - x/2*Log[x])/$eps^2 + 
     ((100 + Pi^2)*(1+x) - 72*x*Log[x] + 6*x*Log[x]^2 + 
       24*(1-x)*PolyLog[2, 1-x])/(24*$eps) + 
     (6*(60 + Pi^2)*(1+x) + 4*Zeta[3]*(13 - 11*x) - 
       (300*x - 4*Pi^2 + 15*Pi^2*x)*Log[x] +
       12*(3*x - (1 - 3*x)*Log0[1-x])*Log[x]^2 - 2*x*Log[x]^3 + 
       24*(6*(1-x) - (1 - 2*x)*Log[x])*PolyLog[2, 1-x] - 
       96*(1-x)*PolyLog[3, 1-x] - 24*(1 - 3*x)*PolyLog[3, x])/24 /. 
     x->m3s/m4s) /; !PossibleZeroQ[m3s*m4s],
  U5[0,0,m3s_,m4s_,m5s_] :> 
   ((-2 + $D)^2*A0[m4s]*A0[m3s]*A0[m5s])/(2*m4s*(m4s - m3s - m5s)*
      (-3 + $D)^2) + A0[m4s]^2*
     (-((m4s - m5s)*(2*m4s^2 - m3s^2 + m3s*m5s - m4s*(m3s + 2*m5s))*
	 (-2 + $D)^2*A0[m3s])/(4*m4s^2*m3s*(4*m4s - m3s)*(m4s - m3s - m5s)*
	(-3 + $D)^2) - ((m4s + m3s - m5s)*(-2 + $D)^2*A0[m5s])/
       (4*m4s^2*(m4s - m3s - m5s)*(-3 + $D)^2)) + 
    (((m4s + m3s - m5s)*(-2 + $D)*A0[m4s])/(2*m4s*(m4s - m3s - m5s)*
	(-3 + $D)) + ((2 - $D)*A0[m3s])/((m4s - m3s - m5s)*(-3 + $D)))*
     T3[0, 0, m5s] + ((-2 + $D)*A0[m4s]*T3[m4s, m3s, m5s])/
     (2*m4s*(-3 + $D)) + ((m4s^2 + (m3s - m5s)^2 - 2*m4s*(m3s + m5s))*
      U4[m4s, m4s, m3s, 0])/((4*m4s - m3s)*(-m4s + m3s + m5s)*
      (-3 + $D)) - ((m4s + m3s - m5s)*U4[m4s, m3s, 0, 0])/
     ((m4s - m3s - m5s)*(-3 + $D)) + 
    ((m4s^2 + (m3s - m5s)^2 - 2*m4s*(m3s + m5s))*U4[m4s, m3s, 0, m4s])/
     ((4*m4s - m3s)*(-m4s + m3s + m5s)*(-3 + $D)) - 
    (2*m3s*U4[m3s, m4s, 0, 0])/((-m4s + m3s + m5s)*(-3 + $D)) - 
    (2*(m4s^2 + (m3s - m5s)^2 - 2*m4s*(m3s + m5s))*U4[m3s, m4s, 0, m4s])/
     ((4*m4s - m3s)*(-m4s + m3s + m5s)*(-3 + $D)) - 
    (m4s*(m4s^2 + (m3s - m5s)^2 - 2*m4s*(m3s + m5s))*
      M1p1121[m4s, 0, m3s, m4s, m5s])/((m4s - m3s - m5s)*(-3 + $D)) + 
    ((m4s - m5s)*(m4s^2 + (m3s - m5s)^2 - 2*m4s*(m3s + m5s))*
      M21121[m4s, 0, m3s, m4s, m5s])/((m4s - m3s - m5s)*(-3 + $D)^2),
  U5[m1s_,0,0,m1s_,m5s_] :>
   -((-2 + $D)^2*A0[m1s]^2*A0[m5s])/(4*m1s^2*(-3 + $D)^2) + 
    ((m1s - m5s)^2*M21121[m1s, 0, 0, m1s, m5s])/(-3 + $D)^2 + 
    ((-2 + $D)*A0[m1s]*T3[0, m1s, m5s])/(m1s*(-3 + $D)) + 
    ((m1s*(24 - 7*$D) + m5s*(-10 + 3*$D))*U4[m1s, 0, 0, m1s])/
     (2*m1s*(-3 + $D)*(-7 + 2*$D)),
  U5[m1s_,m2s_,m2s_,m1s_,m5s_] :>
   -((m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))*(-2 + $D)^2*A0[m1s]^2*
       A0[m2s])/(16*m1s*m2s*(m1s - m2s)*(m1s - m2s - m5s)*(-3 + $D)^2) - 
    ((-2 + $D)^2*A0[m2s]^2*A0[m5s])/(2*m2s*(-m1s + m2s + m5s)*(-3 + $D)^2) + 
    A0[m1s]*(-((3*m1s^2 + 3*m2s^2 - 2*m2s*m5s - m5s^2 - 6*m1s*(m2s - m5s))*
	 (-2 + $D)^2*A0[m2s]^2)/(16*m1s*m2s*(m1s - m2s)*(m1s - m2s - m5s)*
	(-3 + $D)^2) - ((m1s + m2s - m5s)*(-2 + $D)^2*A0[m2s]*A0[m5s])/
       (4*m1s*m2s*(m1s - m2s - m5s)*(-3 + $D)^2)) + 
    ((m1s + m2s - m5s)*(m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))*
      M21121[m1s, m2s, m1s, m2s, m5s])/((m1s - m2s - m5s)*(-3 + $D)^2) + 
    (2*m1s*(m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))*
      M21121[m1s, m2s, m2s, m1s, m5s])/((m1s - m2s - m5s)*(-3 + $D)^2) + 
    (((m1s + m2s - m5s)*(-2 + $D)*A0[m1s])/(2*m1s*(m1s - m2s - m5s)*
	(-3 + $D)) + ((-m1s + 3*m2s + m5s)*(-2 + $D)*A0[m2s])/
       (2*m2s*(-m1s + m2s + m5s)*(-3 + $D)))*T3[m1s, m2s, m5s] + 
    ((-3*m1s^2 + 5*m2s^2 - 6*m2s*m5s + m5s^2 - 2*m1s*(m2s - m5s))*
      U4[m1s, m1s, m2s, m2s])/(4*(m1s - m2s)*(m1s - m2s - m5s)*
      (-3 + $D)) - ((m1s^2 + 9*m2s^2 - 2*m2s*m5s + m5s^2 - 2*m1s*(5*m2s + m5s))*
      U4[m2s, m1s, m1s, m2s])/(4*(m1s - m2s)*(m1s - m2s - m5s)*(-3 + $D)),
  U5[m1s_, m1s_, m3s_, m4s_, 0] | U5[m3s_, m4s_, m1s_, m1s_, 0] :>
   -((-2 + $D)^2*A0[m1s]*(2*m1s*(4*m1s - m3s - 3*m4s)*A0[m3s]*A0[m4s] + 
	A0[m1s]*(2*(4*m1s^2 + m4s*(-m3s + m4s) - m1s*(m3s + 3*m4s))*A0[m3s] + 
	  (-4*m1s*m3s + m3s^2 + 4*m1s*m4s - m4s^2)*A0[m4s])))/
     (2*m1s*(-8*m1s^2*(m3s + m4s*(7 - 2*$D)) + 
       2*m1s*(m3s^2 + m4s^2*(13 - 4*$D) + 2*m3s*m4s*(9 - 2*$D)) + 
       (m3s - m4s)^2*m4s*(-3 + $D))*(-3 + $D)) + 
    ((m3s - m4s)*(8*m1s^2 + m4s*(-m3s + m4s) - 2*m1s*(m3s + 3*m4s))*(-2 + $D)*
      A0[m1s]*T3[0, m3s, m4s])/(2*m1s*(8*m1s^2*(m3s + m4s*(7 - 2*$D)) - 
       2*m1s*(m3s^2 + m4s^2*(13 - 4*$D) + 2*m3s*m4s*(9 - 2*$D)) - 
       (m3s - m4s)^2*m4s*(-3 + $D))) + 
    (((m3s - m4s)^2*m4s - 8*m1s^2*(m3s + m4s) + 
       2*m1s*(m3s^2 + 8*m3s*m4s - m4s^2))*U4[m1s, m1s, m3s, m4s])/
     (8*m1s^2*(m3s + m4s*(7 - 2*$D)) - 2*m1s*(m3s^2 + m4s^2*(13 - 4*$D) + 
	2*m3s*m4s*(9 - 2*$D)) - (m3s - m4s)^2*m4s*(-3 + $D)) - 
    (4*m1s*m3s*(4*m1s - m3s - 3*m4s)*U4[m3s, m1s, m1s, m4s])/
     (8*m1s^2*(m3s + m4s*(7 - 2*$D)) - 2*m1s*(m3s^2 + m4s^2*(13 - 4*$D) + 
	2*m3s*m4s*(9 - 2*$D)) - (m3s - m4s)^2*m4s*(-3 + $D)) + 
    (4*m1s*m4s*(-m3s + m4s)*(16*m1s^2 + (m3s - m4s)^2 - 8*m1s*(m3s + m4s))*
      M21121[m1s, m1s, m3s, m4s, 0])/((-8*m1s^2*(m3s + m4s*(7 - 2*$D)) + 
       2*m1s*(m3s^2 + m4s^2*(13 - 4*$D) + 2*m3s*m4s*(9 - 2*$D)) + 
       (m3s - m4s)^2*m4s*(-3 + $D))*(-3 + $D)),
  U5[m1s_,m2s_,m3s_,m4s_,m5s_] :>
   ((m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))*(-2 + $D)^2*
      ((4*(m1s^2 + m2s^2 - 3*m3s^2 + 2*m2s*(m3s - m4s) + 2*m3s*m4s + m4s^2 - 
	  2*m1s*(m2s - m3s + m4s))*A0[m2s]*A0[m3s]*A0[m4s])/
	(m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
	 (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
	 m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
	 4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
           m2s*(m3s^2 - 10*m3s*m4s + m4s^2))) - 
       (2*(m1s^2 + m2s^2 - 2*m2s*m3s + m3s^2 - 2*m1s*(m2s + m3s - m4s) + 
	  2*m2s*m4s + 2*m3s*m4s - 3*m4s^2)*(m3s + m4s - m5s)*A0[m2s]*A0[m3s]*
	 A0[m4s])/(m4s*(m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
	  (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
	  m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
	  4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
            m2s*(m3s^2 - 10*m3s*m4s + m4s^2)))) - 
       (2*A0[m1s]*((m1s^3 + m2s^3 + (m3s - m4s)^3 - m2s^2*(m3s + 3*m4s) - 
            m1s^2*(m2s + m3s + 3*m4s) - m2s*(m3s^2 + 2*m3s*m4s - 3*m4s^2) - 
            m1s*(m2s^2 + m3s^2 + 2*m3s*m4s - 3*m4s^2 + 2*m2s*(-5*m3s + m4s)))*
           A0[m3s]*A0[m4s] + A0[m2s]*((m1s^3 - m2s^3 + 3*m2s^2*(m3s + m4s) + 
              (m3s - m4s)^2*(m3s + m4s) - m1s^2*(3*m2s + m3s + m4s) - 
              m2s*(3*m3s^2 + 2*m3s*m4s + 3*m4s^2) + m1s*(3*m2s^2 - m3s^2 + 
        	10*m3s*m4s - m4s^2 - 2*m2s*(m3s + m4s)))*A0[m3s] - 
            2*m3s*(-3*m1s^2 + m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s) + 
              2*m1s*(m2s + m3s + m4s))*A0[m4s])))/
	(m1s*(m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
	  (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
	  m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
	  4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
            m2s*(m3s^2 - 10*m3s*m4s + m4s^2)))) + 
       ((m3s + m4s - m5s)*A0[m1s]*((m1s^3 + m2s^3 - (m3s - m4s)^3 - 
            m2s^2*(3*m3s + m4s) - m1s^2*(m2s + 3*m3s + m4s) + 
            m2s*(3*m3s^2 - 2*m3s*m4s - m4s^2) - m1s*(m2s^2 - 3*m3s^2 + 
              2*m2s*(m3s - 5*m4s) + 2*m3s*m4s + m4s^2))*A0[m3s]*A0[m4s] + 
	  A0[m2s]*(-2*m4s*(-3*m1s^2 + m2s^2 + (m3s - m4s)^2 - 
              2*m2s*(m3s + m4s) + 2*m1s*(m2s + m3s + m4s))*A0[m3s] + 
            (m1s^3 - m2s^3 + 3*m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
              m1s^2*(3*m2s + m3s + m4s) - m2s*(3*m3s^2 + 2*m3s*m4s + 3*m4s^2) + 
              m1s*(3*m2s^2 - m3s^2 + 10*m3s*m4s - m4s^2 - 2*m2s*(m3s + m4s)))*
             A0[m4s])))/(m1s*m4s*(m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
	  (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
	  m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
	  4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
            m2s*(m3s^2 - 10*m3s*m4s + m4s^2)))) - 
       (2*A0[m2s]*(2*m4s*A0[m3s] - (m3s + m4s - m5s)*A0[m4s])*A0[m5s])/
	(m4s*(m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))) + 
       (A0[m1s]*(2*m4s*A0[m3s] - (m3s + m4s - m5s)*A0[m4s])*
	 ((m1s - m2s + m5s)*A0[m2s] + (m1s + m2s - m5s)*A0[m5s]))/
	(m1s*m4s*(m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s)))))/
     (4*(-m1s + m2s + m5s)*(m3s - m4s + m5s)*(-3 + $D)^2) + 
    ((-2 + $D)*(2*m4s*A0[m3s] - (m3s + m4s - m5s)*A0[m4s])*T3[m1s, m2s, m5s])/
     (2*m4s*(m3s - m4s + m5s)*(-3 + $D)) - 
    ((-2 + $D)*((m1s + m2s - m5s)*A0[m1s] - 2*m1s*A0[m2s])*T3[m3s, m4s, m5s])/
     (2*m1s*(-m1s + m2s + m5s)*(-3 + $D)) - 
    ((m3s^2 + (m4s - m5s)^2 - 2*m3s*(m4s + m5s))*
      (m1s^4 + (m2s^3 + (m3s - m4s)^3 - m2s^2*(m3s + 3*m4s) - 
	 m2s*(m3s^2 + 2*m3s*m4s - 3*m4s^2))*(m2s - m5s) + 
       m1s^3*(4*m2s - m3s - 3*m4s - m5s) + 
       m1s^2*(-10*m2s^2 - (m3s + 3*m4s)*(m3s - m4s - m5s) + 
	 m2s*(17*m3s - 13*m4s + m5s)) + 
       m1s*(4*m2s^3 + m2s^2*(17*m3s - 13*m4s + m5s) + 
	 (m3s - m4s)*(m3s^2 + m3s*(-2*m4s + m5s) + m4s*(m4s + 3*m5s)) - 
	 2*m2s*(7*m3s^2 - m4s*(5*m4s + m5s) + m3s*(-2*m4s + 5*m5s))))*
      U4[m1s, m2s, m3s, m4s])/((m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
       (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
       m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
       4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
	 m2s*(m3s^2 - 10*m3s*m4s + m4s^2)))*(m1s - m2s - m5s)*(m3s - m4s + m5s)*
      (-3 + $D)) + (2*m2s*(m3s^2 + (m4s - m5s)^2 - 2*m3s*(m4s + m5s))*
      (2*m1s^3 + 2*m2s^3 + m2s^2*(m3s - 5*m4s - m5s) - 
       2*m2s*(m3s - m4s)*(2*m3s + 2*m4s + m5s) - 
       m1s^2*(2*m2s - m3s + 5*m4s + m5s) + (m3s - m4s)*
	(m3s^2 - 2*m3s*m4s + m4s^2 + 3*m3s*m5s + m4s*m5s) - 
       2*m1s*(m2s^2 - m2s*(7*m3s - 3*m4s + m5s) + 
	 (m3s - m4s)*(2*m3s + 2*m4s + m5s)))*U4[m2s, m1s, m3s, m4s])/
     ((m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
       (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
       m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
       4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
	 m2s*(m3s^2 - 10*m3s*m4s + m4s^2)))*(m1s - m2s - m5s)*(m3s - m4s + m5s)*
      (-3 + $D)) + (2*m3s*(m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))*
      (-m1s^3 + m2s^3 + (m3s - m4s)^2*(2*m3s + 2*m4s - m5s) + 
       m1s^2*(3*m2s + 4*m3s + 4*m4s - m5s) + m2s^2*(-4*m3s - 4*m4s + 3*m5s) + 
       m2s*(m3s^2 + 14*m3s*m4s + m4s^2 - 2*m3s*m5s - 2*m4s*m5s) - 
       m1s*(3*m2s^2 + 5*m3s^2 + 6*m3s*m4s + 5*m4s^2 + 2*m2s*m5s - 2*m3s*m5s - 
	 2*m4s*m5s))*U4[m3s, m1s, m2s, m4s])/
     ((m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
       (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
       m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
       4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
	 m2s*(m3s^2 - 10*m3s*m4s + m4s^2)))*(m1s - m2s - m5s)*(m3s - m4s + m5s)*
      (-3 + $D)) + ((m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))*
      (-(m1s^2*(3*m3s^2 + m3s*(10*m4s - 3*m5s) + 3*m4s*(m4s - m5s) + 
	  3*m2s*(m3s + m4s - m5s))) + m1s^3*(m3s + m4s - m5s) - 
       m2s^3*(m3s + m4s - m5s) - (m3s - m4s)^2*(m3s^2 + 6*m3s*m4s + m4s^2 - 
	 m3s*m5s - m4s*m5s) + m2s^2*(m3s^2 + 14*m3s*m4s + m4s^2 - m3s*m5s - 
	 m4s*m5s) + m2s*(m3s^3 + m4s^2*(m4s - m5s) - m3s^2*(17*m4s + m5s) + 
	 m3s*m4s*(-17*m4s + 10*m5s)) + m1s*(3*m3s^3 + m3s^2*(13*m4s - 3*m5s) + 
	 m3s*m4s*(13*m4s - 2*m5s) + 3*m4s^2*(m4s - m5s) + 
	 3*m2s^2*(m3s + m4s - m5s) + 2*m2s*(m3s^2 + m4s*(m4s - m5s) - 
           m3s*(2*m4s + m5s))))*U4[m4s, m1s, m2s, m3s])/
     ((m1s^4 - 4*m1s^3*(m2s + m3s + m4s) + 
       (m2s^2 + (m3s - m4s)^2 - 2*m2s*(m3s + m4s))^2 + 
       m1s^2*(6*m2s^2 + 6*m3s^2 + 4*m3s*m4s + 6*m4s^2 + 4*m2s*(m3s + m4s)) - 
       4*m1s*(m2s^3 - m2s^2*(m3s + m4s) + (m3s - m4s)^2*(m3s + m4s) - 
	 m2s*(m3s^2 - 10*m3s*m4s + m4s^2)))*(m1s - m2s - m5s)*(m3s - m4s + m5s)*
      (-3 + $D)) + ((m1s^2 + (m2s - m5s)^2 - 2*m1s*(m2s + m5s))*
      (m3s^2 + (m4s - m5s)^2 - 2*m3s*(m4s + m5s))*M21121[m1s,m2s,m3s,m4s,m5s])/
     ((-m1s + m2s + m5s)*(m3s - m4s + m5s)*(-3 + $D)^2),
  U6[m1s_,m2s_,m3s_,m4s_,m5s_,m6s_] :> 2*Zeta[3]*(1/$eps - 3*Log[m6s] + 3) -
      17*Zeta[4] - 4*Zeta[2]*Log[2]^2 + 2/3*Log[2]^4 + 16*PolyLog[4, 1/2] -
      4*Im[PolyLog[2, Exp[I*Pi/3]]]^2 + U6sub[m1s,m2s,m3s,m4s,m5s,m6s] /; 
     !PossibleZeroQ[m6s]
};

Log0[0] := 0;
Log0[lex_ /; NumberQ[lex]] := Piecewise[{{0, Abs[lex] < 1*^-50}}, Log[lex]];

Get[DirectoryName[$Input] <> "uself.m"];

PrepInt[expr_] := Module[{expre},
  expre = expr //. usrepl //. tsrepl;
  expre = expre /. usort /. urepl;
  expre = Collect[expre, {U5asub[__],U5bsub[__],U5csub[__],
  		          U6asub[__],U6bsub[__],U6csub[__],
  		          U5a1sub[__],U5a2sub[__],U5b1sub[__],U5b2sub[__],
			  U5c1sub[__],U6m1sub[__],U6m3sub[__],U6n2sub[__],
  		          U6sub[__],M21121[__],U4sub[__],
  			  T3asub[__],T3adelta[__],T3a1sub[__],T3a1delta[__],
			  T4asub[__],T4adelta[__],T4a1sub[__],T4a1delta[__],
			  U4[__],T3[__],B0fin[__],B0delta[__],B0deltadelta[__]}];
  expre = expre /. usort /. urepl //. trepl /. $D -> 4-2*$eps;
  expre = expre //. (xx_)^nn_ :> Normal[Series[xx^nn, {$eps,0,3}]] /;
  				!FreeQ[nn, $eps] /.
		    (xx_ + yy_*$eps)^nn_ :> xx^nn *
		      (1 + $eps*nn*yy/xx + $eps^2*nn*(nn-1)/2*(yy/xx)^2 + 
		       $eps^3*nn*(nn-1)*(nn-2)/6*(yy/xx)^3) /;
		     (nn<0) /.
		   gamm_Gamma :> Normal[Series[gamm, {$eps,0,3}]] /;
		   		 !FreeQ[gamm, $eps];
  expre = expre /. pp_PolyLog :> Simplify[pp] /.
  		   ll_Log :> Simplify[ll];
  expre = Collect[expre, {U5asub[__],U5bsub[__],U5csub[__],
  		          U6asub[__],U6bsub[__],U6csub[__],
  		          U5a1sub[__],U5a2sub[__],U5b1sub[__],U5b2sub[__],
			  U5c1sub[__],U6m1sub[__],U6m3sub[__],U6n2sub[__],
  		          U6sub[__],M21121[__],U4sub[__],
  			  T3asub[__],T3adelta[__],T3a1sub[__],T3a1delta[__],
			  T4asub[__],T4adelta[__],T4a1sub[__],T4a1delta[__],
			  T3Idelta[__],B0fin[__],B0delta[__],B0deltadelta[__](*,PolyLog[_,_]*)}];
  expre = Map[Series[#, {$eps,0,0}]&, List @@ (expre + $dummy)];
  expre = Collect[Plus @@ Map[Normal, expre /. $dummy->0], $eps];
  Return[expre];
(*  Return[Map[Series[#, {$eps,0,0}]&, expre]];
*)
(*  expre = Map[Simplify, expre];
  Return[Series[expre, {$eps,0,0}]]*)
];


$Directory = StringReplace[DirectoryName[$InputFileName], "mcode" -> "ccode"];
$FileIn = "u456.in";
$FileOut = "u456.out";
$Silent = True;

NIntRepl = { T3asub->T3a, T3a1sub->T3a1, T4asub->T4a, T4a1sub->T4a1, 
	     U4sub->U4, U4sub0->U40, M21121->U5, U6sub->U6,
             M1p1121[m4s_, 0, m3s_, m4s_, m5s_] :> U50[m3s, m4s, m5s],
	     U4asub->U4a, U5asub->U5a, U5bsub->U5b, U5csub->U5c, U6asub->U6a, 
	     U6bsub->U6b, U6csub->U6c, U6msub->U6m, U6nsub->U6n, U7msub->U7m, 
	     U4a1sub->U4a1, U4a2sub->U4a2, U4a3sub->U4a3,
	     U5a1sub->U5a1, U5a2sub->U5a2, U5b1sub->U5b1, U5b2sub->U5b2,
	     U5c1sub->U5c1, U6m1sub->U6m1, U6m3sub->U6m3, U6n2sub->U6n2 };
NIntList = { B0fin[__], T3a[__], T3a1[__], T4a[__], T4a1[__], T5a[__], 
	     U4[__], U40[__], U5[__], U50[__], U6[__],
	     U4a[__], U5a[__], U5b[__], U5c[__], U6a[__], U6b[__], U6c[__],
	     U6m[__], U6n[__], U7m[__], U7a[__], U8a[__],
	     U4a1[__], U4a2[__], U4a3[__], U5a1[__], U5a2[__], U5b1[__], 
	     U5b2[__], U5c1[__], U6m1[__], U6m2[__], U6m3[__], U6n1[__], 
	     U6n2[__], U7a1[__], U7a2[__] };

UCall[inexpr_] := Module[{expr2,tints,tlist,tnums},
  expr2 = inexpr /. trepl2 /. NIntRepl;
  tints = Union[Extract[expr2, Position[expr2, Alternatives @@ NIntList]]];
  tlist = Map[Prepend[List @@ #, Head[#]]&, tints];
  If[Length[tlist] > 0,
    Export[$FileIn, tlist, "Table"];
    If[$Silent,
      Run[$Directory <> "ucall " <> $FileIn <> " " <> $FileOut <> " >/dev/null"],
      Run[$Directory <> "ucall " <> $FileIn <> " " <> $FileOut]
    ];
    tnums = ReadList[$FileOut, {Number,Number}];
    tnums = Map[Complex @@ Chop[#] &, tnums];
    tints = Map[Rule @@ # &, Transpose[{tints, tnums}]];
  ];
  Return[expr2 /. tints];
];

UCallE[inexpr_] := Module[{expr2,tints,tlist,tnums,errcnt=0},
  expr2 = inexpr /. trepl2 /. NIntRepl;
  tints = Union[Extract[expr2, Position[expr2, Alternatives @@ NIntList]]];
  tlist = Map[Prepend[List @@ #, Head[#]]&, tints];
  If[Length[tlist] > 0,
    Export[$FileIn, tlist, "Table"];
    If[$Silent,
      Run[$Directory <> "ucall " <> $FileIn <> " " 
      		<> $FileOut <> " -e >/dev/null"],
      Run[$Directory <> "ucall " <> $FileIn <> " " 
      		<> $FileOut <> " -e"]
    ];
    tnums = ReadList[$FileOut, {Number,Number,Number}];
    tnums = Map[Complex @@ Chop[Take[#,2]] + #[[3]]*pm[++errcnt] &, tnums];
    tints = Map[Rule @@ # &, Transpose[{tints, tnums}]];
  ];
  Return[expr2 /. tints];
];
